/*
  This is an Arduino version of the LMR Start Here robot.
  The original Picaxe version can be found at:
  http://letsmakerobots.com/start
  This version was written for the Arduino Mega
  by ignoblegnome: http://letsmakerobots.com/user/5293
*/

#define DEBUG 0    // set to 1 to print to serial monitor, 0 to disable
#include <Servo.h>

Servo headservo;  // creates servo object

// Constants
int sharppin = 54; // analog pin for Sharp IR (1st analog pin for Mega is 54)
int ledpin = 13;    // onboard LED
int headservopin = 9;   // define servo pin
int leftmotorpin1 = 7;  // define your motor pins
int leftmotorpin2 = 6;
int rightmotorpin1 = 5;
int rightmotorpin2 = 4;
int dangerlevel = 600;   // how far away should things be, before we react?
int turn = 300;          // this sets how much we should turn
int servo_turn = 700;    // this sets how long we should wait for the serov to turn before we measure distance

// Variables
int currDist = 0;    // holds the current distance
int compDist = 0;  // holds a distance to compare currDist to

void setup() {
  pinMode(13, OUTPUT);   
  for (int pinindex = 0; pinindex < 14; pinindex++) {
     pinMode(pinindex, OUTPUT);  // set pins 0 to 13 as outputs
  }
  
  Serial.begin(9600); // start serial monitor

  for (int pinindex = 55; pinindex < 70; pinindex++) {
     digitalWrite(pinindex, HIGH);  // set all the internal pullups for unused analog pins
  }
  headservo.attach(headservopin);  // attaches servo to pin
  headservo.write(70);  //  set servo to face forward
}

void loop() {
  currDist = analogRead(sharppin);
  if(currDist < dangerlevel) {
    nodanger();      // if no danger, drive ahead
  }
  else {
    whichway();      // if obstacle ahead then decide with way is better
  }
//  delay(500);
  if(DEBUG){
    Serial.print("Current Distance: ");
    Serial.println(currDist);
  }
}

void nodanger() {      // drive forward
  digitalWrite(ledpin, HIGH);
  digitalWrite(leftmotorpin1, HIGH);
  digitalWrite(leftmotorpin2, LOW);
  digitalWrite(rightmotorpin1, HIGH);
  digitalWrite(rightmotorpin2, LOW);
  return;
}  

void whichway() {
  digitalWrite(ledpin, LOW);
  totalhalt();    // first stop!
  
  lturn();        // look left
  delay(servo_turn);  // wait for servo to stop turning
  int lDist = analogRead(sharppin);   // check distance
  totalhalt();      // totalhalt also centers the servo

  rturn();        // look right
  delay(servo_turn);  // wait for servo to stop turning
  int rDist = analogRead(sharppin);   // check distance
  totalhalt();      // totalhalt also centers the servo

  if(lDist < rDist) {  // lower analog value is further distance
    body_lturn();
  }
    else {
    body_rturn();
  }
  return;
}  

void totalhalt() {
  digitalWrite(leftmotorpin1, LOW);
  digitalWrite(leftmotorpin2, LOW);
  digitalWrite(rightmotorpin1, LOW);
  digitalWrite(rightmotorpin2, LOW);
  headservo.write(70);  //  set servo to face forward
  delay(1000);
}  

void lturn() {
  headservo.write(20);  // turn the servo left
}  

void rturn() {
  headservo.write(120);  // turn the servo right
}  

void body_lturn() {
  digitalWrite(leftmotorpin1, LOW);
  digitalWrite(leftmotorpin2, HIGH);
  digitalWrite(rightmotorpin1, HIGH);
  digitalWrite(rightmotorpin2, LOW);
  delay(turn);
  totalhalt();
}  

void body_rturn() {
  digitalWrite(leftmotorpin1, HIGH);
  digitalWrite(leftmotorpin2, LOW);
  digitalWrite(rightmotorpin1, LOW);
  digitalWrite(rightmotorpin2, HIGH);
  delay(turn);
  totalhalt();
}  
